@@ -1,5 +1,5 @@
 /*  Rational - Rational number class with overflow detection
-    Copyright (C) 2012-2017 Antonio Diaz Diaz.
+    Copyright (C) 2012-2018 Antonio Diaz Diaz.
 
     This program is free software. Redistribution and use in source and
     binary forms, with or without modification, are permitted provided
@@ -39,7 +39,7 @@ namespace {
 
 const char * const Program_name = "Rational";
 const char * const program_name = "rational";
-const char * const program_year = "2017";
+const char * const program_year = "2018";
 const char * invocation_name = 0;
 
 
@@ -48,7 +48,7 @@ void show_help()
   std::printf( "%s - Rational number class with overflow detection.\n", Program_name );
   std::printf( "See the source file 'main.cc' to learn how to use %s in\n", Program_name );
   std::printf( "your own programs.\n"
-               "\nUsage: %s [options]\n", invocation_name );
+               "\nUsage: %s [options] [arguments]\n", invocation_name );
   std::printf( "\nOptions:\n"
                "  -h, --help                   display this help and exit\n"
                "  -V, --version                output version information and exit\n"
@@ -57,6 +57,7 @@ void show_help()
                "  -s, --sqrt=<n>               calculate approximations of sqrt(n)\n"
                "  -t, --test=<level>           run tests at given level (1-9)\n"
                "  -v, --verbose                verbose operation\n"
+               "The option -s, --sqrt accepts a rational argument.\n"
                "\nReport bugs to arg-parser-bug@nongnu.org\n"
                "Rational home page: http://www.nongnu.org/arg-parser/rational.html\n" );
   }
@@ -101,7 +102,7 @@ void approximate_pi()
   int max_len = 1;
   int n = 3, d = 1, dif = 0;
 
-  std::printf( "target == %s\n", target.c_str() );
+  std::printf( "target = %s\n", target.c_str() );
   std::fputs( "correct\ndecimals                  value\n", stdout );
   while( n <= INT_MAX - 3 )
     {
@@ -129,34 +130,40 @@ void approximate_pi()
   }
 
 
-void approximate_sqrt( const int itarget )
+void approximate_sqrt( const Rational target )
   {
-  const Rational target( itarget );
-  Rational best_value( 1 ), best_error( itarget - 1 );
+  // works for targets from 1/65536 to 462409
+  Rational best_error( target );		// for 0 < target < 1
+  if( target >= 46226 ) best_error = 10;
+  else if( target >= 1 ) best_error.assign( 1, ( target >= 15047 ) ? 1 : 10 );
   const int prec = 10;
   int n = 1, d = 1;
 
-  std::printf( "target == sqrt( %d )\n", itarget );
-  std::fputs( "        error               value^2          value\n", stdout );
-  while( n < INT_MAX )
+  if( target.denominator() == 1 )
+    std::printf( "target = sqrt( %s )\n", target.to_decimal().c_str() );
+  else
+    std::printf( "target = sqrt( %s ) (%s)\n",
+      target.to_decimal( 1, -2 * prec ).c_str(), target.to_fraction().c_str() );
+  std::fputs( "        error                value^2                  value\n", stdout );
+  while( true )
     {
     const Rational value( n, d );
     Rational value2( value ); value2 *= value2;
+    if( value2.error() ) break;
     Rational error( value2 ); error -= target;
-    if( value2.error() || error.error() ) break;
-    if( error.abs() < best_error && error.abs() < 1 )
+    if( !error.error() && error.abs() < best_error )
       {
-      best_value = value; best_error = error.abs();
-      std::printf( "%-21s    %-12s    %-12s (%s)\n",
-                   error.to_decimal( 2, -( prec + 8 ) ).c_str(),
+      best_error = error.abs();
+      std::printf( "%-23s    %-19s    %-12s (%s)\n",
+                   error.to_decimal( 2, -2 * prec ).c_str(),
                    value2.to_decimal( 1, -prec ).c_str(),
                    value.to_decimal( 1, -prec ).c_str(),
                    value.to_fraction().c_str() );
       std::fflush( stdout );
       if( best_error == 0 ) break;
       }
-    if( error < 0 ) ++n;
-    else ++d;
+    if( value2 < target ) { if( ++n >= INT_MAX ) break; }
+    else if( ++d >= INT_MAX ) break;
     }
   }
 
@@ -319,13 +326,38 @@ int run_tests( const int level, const bool verbose )
       retval = 1; }
   }
 
+  // relational test
+  {
+  const Rational rm1( -1 );
+  const Rational r0;
+  const Rational r1( 1 );
+  const Rational r2( 2 );
+  if( rm1 < rm1 || r0 < r0 || r1 < r1 || r2 < r2 )
+    { std::fputs( "error: bad relational result for 'a < a'.\n", stderr );
+      retval = 1; }
+  if( r2 < r1 || r1 < r0 || r0 < rm1 )
+    { std::fputs( "error: bad relational result for 'a < b'.\n", stderr );
+      retval = 1; }
+  if( r2 <= r1 || r1 <= r0 || r0 <= rm1 )
+    { std::fputs( "error: bad relational result for 'a <= b'.\n", stderr );
+      retval = 1; }
+  if( rm1 > rm1 || r0 > r0 || r1 > r1 || r2 > r2 )
+    { std::fputs( "error: bad relational result for 'a > a'.\n", stderr );
+      retval = 1; }
+  if( rm1 > r0 || r0 > r1 || r1 > r2 )
+    { std::fputs( "error: bad relational result for 'a > b'.\n", stderr );
+      retval = 1; }
+  if( rm1 >= r0 || r0 >= r1 || r1 >= r2 )
+    { std::fputs( "error: bad relational result for 'a >= b'.\n", stderr );
+      retval = 1; }
+  }
 
-  const int sum_delta = 1000 * level * level * level;
+  const int sum_delta = ( 10 - level ) * 1000;
   const int sum_limit = INT_MAX / 2;
   const int mul_limit = 46340;		// sqrt( 1 << 31 )
 
   // addition/comparison test
-  for( int n = sum_limit - sum_delta; n <= sum_limit; ++n )
+  for( int n = 0; n <= sum_limit; n += sum_delta )
     {
     const int n2 = 2 * n;
     const Rational r1( n2, 2 );
@@ -519,7 +551,7 @@ int run_tests( const int level, const bool verbose )
 int main( const int argc, const char * const argv[] )
   {
   int level = 0;
-  int sqrt_target = 0;
+  Rational sqrt_target;
   bool pi_flag = false;
   bool verbose = false;
   invocation_name = argv[0];
@@ -550,9 +582,8 @@ int main( const int argc, const char * const argv[] )
       case 'h': show_help(); return 0;
       case 'p': pi_flag = true; break;
       case 'q': verbose = false; break;
-      case 's': sqrt_target = strtoul( arg, 0, 0 );
-                if( sqrt_target < 2 )
-                  { show_error( "Bad sqrt target", 0, true ); return 1; }
+      case 's': if( !sqrt_target.parse( arg ) || sqrt_target <= 0 )
+                  { show_error( "Bad sqrt argument", 0, true ); return 1; }
                 break;
       case 't': if( std::isdigit( *arg ) && *arg != '0' && !arg[1] )
                   level = *arg - '0';
